@@ -1,12 +1,14 @@
 from __future__ import annotations
 
 import sys
+from contextlib import AbstractContextManager
 from copy import deepcopy
-from typing import TYPE_CHECKING, Any, TypeVar
+from types import TracebackType
+from typing import TYPE_CHECKING, Any, Generic, TypeVar
 
 from typing_extensions import Self
 
-from ..types import MISSING, FuncExceptT, MissingT, SupportsString
+from ..types import MISSING, FuncExcept, MissingT, SupportsString
 
 __all__ = [
     "CustomError",
@@ -34,7 +36,7 @@ class CustomErrorMeta(type):
     """Custom base exception meta class."""
 
     def __new__(cls: type[SelfCErrorMeta], *args: Any) -> SelfCErrorMeta:
-        return CustomErrorMeta.setup_exception(type.__new__(cls, *args))  # pyright: ignore[reportReturnType]
+        return CustomErrorMeta.setup_exception(super().__new__(cls, *args))
 
     @staticmethod
     def setup_exception(exception: SelfCErrorMeta, override: str | ExceptionError | None = None) -> SelfCErrorMeta:
@@ -78,7 +80,7 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
     """Custom base exception class."""
 
     def __init__(
-        self, message: SupportsString | None = None, func: FuncExceptT | None = None, reason: Any = None, **kwargs: Any
+        self, message: SupportsString | None = None, func: FuncExcept | None = None, reason: Any = None, **kwargs: Any
     ) -> None:
         """
         Instantiate a new exception with pretty printing and more.
@@ -105,8 +107,8 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
     def __call__(
         self,
         message: SupportsString | None | MissingT = MISSING,
-        func: FuncExceptT | None | MissingT = MISSING,
-        reason: SupportsString | FuncExceptT | None | MissingT = MISSING,
+        func: FuncExcept | None | MissingT = MISSING,
+        reason: SupportsString | FuncExcept | None | MissingT = MISSING,
         **kwargs: Any,
     ) -> Self:
         """
@@ -166,12 +168,69 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
         else:
             reason = ""
 
-        return f"{func_header}{self.message!s}{reason}".format(**self.kwargs).strip()
+        out = f"{func_header}{self.message!s}{reason}".format(**self.kwargs).strip()
+
+        if sys.version_info < (3, 13) and hasattr(self, "__notes__"):
+            out += "\n" + "\n".join(self.__notes__)
+
+        return out
+
+    if sys.version_info < (3, 11):
+        __notes__: list[str]
+
+        def add_note(self, note: str, /) -> None:
+            if not hasattr(self, "__notes__"):
+                self.__notes__ = []
+
+            self.__notes__.append(note)
+
+    @classmethod
+    def catch(cls) -> CatchError[Self]:
+        """
+        Create a context manager that catches exceptions of this class type.
+
+        Returns:
+            CatchError[Self]: A context manager that will catch and store exceptions of type `cls`
+                when used in a `with` block.
+        """
+        return CatchError(cls)
 
 
 SelfError = TypeVar("SelfError", bound=CustomError)
 
 
+class CatchError(AbstractContextManager["CatchError[SelfError]"], Generic[SelfError]):
+    """
+    Context manager for catching a specific exception type.
+    """
+
+    error: SelfError | None
+    """The caught exception instance, if any."""
+    tb: TracebackType | None
+    """The traceback object associated with the caught exception."""
+
+    def __init__(self, error: type[SelfError]) -> None:
+        self.error = None
+        self.tb = None
+        self._to_catch_error = error
+
+    def __enter__(self) -> Self:
+        return self
+
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
+    ) -> bool | None:
+        if isinstance(exc_value, self._to_catch_error):
+            self.error = exc_value
+            self.tb = traceback
+            return True
+
+        return None
+
+
 class CustomValueError(CustomError, ValueError):
     """Thrown when a specified value is invalid."""
 
